@@ -12,6 +12,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/pkg/domaintags"
 	"github.com/StackExchange/dnscontrol/v4/pkg/prettyzone"
 	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/rtypecontrol"
 
 	"github.com/urfave/cli/v3"
 )
@@ -79,7 +80,7 @@ EXAMPLES:
 
 // check-creds foo bar
 // is the same as
-// get-zones --format=nameonly foo bar all
+// get-zones --format=nameonly foo bar all.
 var _ = cmd(catUtils, func() *cli.Command {
 	var args GetZoneArgs
 	return &cli.Command{
@@ -222,6 +223,7 @@ func GetZone(args GetZoneArgs) error {
 		if err != nil {
 			return fmt.Errorf("failed GetZone gzr: %w", err)
 		}
+		rtypecontrol.FixLegacyRecords(&recs) // Call this after GetZoneRecords() to fix providers that haven't been updated for RecordConfigV2.
 		zoneRecs[i] = recs
 	}
 
@@ -278,6 +280,28 @@ func GetZone(args GetZoneArgs) error {
 			if defaultTTL != models.DefaultTTL && defaultTTL != 0 {
 				o = append(o, fmt.Sprintf("DefaultTTL(%d)", defaultTTL))
 			}
+
+			// Check if any records have comments or tags, and add management flags if so
+			hasComments := false
+			hasTags := false
+			for _, rec := range recs {
+				if rec.Metadata["cloudflare_comment"] != "" {
+					hasComments = true
+				}
+				if rec.Metadata["cloudflare_tags"] != "" {
+					hasTags = true
+				}
+				if hasComments && hasTags {
+					break
+				}
+			}
+			if hasComments {
+				o = append(o, "CF_MANAGE_COMMENTS // opt into comments syncing")
+			}
+			if hasTags {
+				o = append(o, "CF_MANAGE_TAGS // opt into tags syncing")
+			}
+
 			for _, rec := range recs {
 				if (rec.Type == "CNAME") && (rec.Name == "@") {
 					o = append(o, "// NOTE: CNAME at apex may require manual editing.")
@@ -309,19 +333,33 @@ func GetZone(args GetZoneArgs) error {
 
 		case "tsv":
 			for _, rec := range recs {
-				cfproxy := ""
+				cfmeta := ""
 				if cp, ok := rec.Metadata["cloudflare_proxy"]; ok {
 					if cp == "true" {
-						cfproxy = "\tcloudflare_proxy=true"
+						cfmeta += ",cloudflare_proxy=true"
 					}
 				}
+				if cf, ok := rec.Metadata["cloudflare_cname_flatten"]; ok {
+					if cf == "on" {
+						cfmeta += ",cloudflare_cname_flatten=on"
+					}
+				}
+				if comment := rec.Metadata["cloudflare_comment"]; comment != "" {
+					cfmeta += ",cloudflare_comment=" + comment
+				}
+				if tags := rec.Metadata["cloudflare_tags"]; tags != "" {
+					cfmeta += ",cloudflare_tags=" + tags
+				}
+				if cfmeta != "" {
+					cfmeta = "\t" + cfmeta[1:] // Remove leading comma, add tab
+				}
 
 				ty := rec.Type
 				if rec.Type == "UNKNOWN" {
 					ty = rec.UnknownTypeName
 				}
 				fmt.Fprintf(w, "%s\t%s\t%d\tIN\t%s\t%s%s\n",
-					rec.NameFQDN, rec.Name, rec.TTL, ty, rec.GetTargetCombinedFunc(nil), cfproxy)
+					rec.NameFQDN, rec.Name, rec.TTL, ty, rec.GetTargetCombinedFunc(nil), cfmeta)
 			}
 
 		default:
@@ -358,6 +396,29 @@ func formatDsl(rec *models.RecordConfig, defaultTTL uint32) string {
 		}
 	}
 
+	cfflatten := ""
+	if cf, ok := rec.Metadata["cloudflare_cname_flatten"]; ok {
+		if cf == "on" {
+			cfflatten = ", CF_CNAME_FLATTEN_ON"
+		}
+	}
+
+	cfcomment := ""
+	if comment := rec.Metadata["cloudflare_comment"]; comment != "" {
+		cfcomment = fmt.Sprintf(", CF_COMMENT(%s)", jsonQuoted(comment))
+	}
+
+	cftags := ""
+	if tags := rec.Metadata["cloudflare_tags"]; tags != "" {
+		// Convert comma-separated tags to CF_TAGS("tag1", "tag2", ...)
+		tagList := strings.Split(tags, ",")
+		quotedTags := make([]string, len(tagList))
+		for i, tag := range tagList {
+			quotedTags[i] = fmt.Sprintf("%q", tag)
+		}
+		cftags = ", CF_TAGS(" + strings.Join(quotedTags, ", ") + ")"
+	}
+
 	switch rec.Type { // #rtype_variations
 	case "CAA":
 		return makeCaa(rec, ttlop)
@@ -410,7 +471,7 @@ func formatDsl(rec *models.RecordConfig, defaultTTL uint32) string {
 		target = `"` + target + `"`
 	}
 
-	return fmt.Sprintf(`%s("%s", %s%s%s)`, rec.Type, rec.Name, target, cfproxy, ttlop)
+	return fmt.Sprintf(`%s("%s", %s%s%s%s%s%s)`, rec.Type, rec.Name, target, cfproxy, cfflatten, cfcomment, cftags, ttlop)
 }
 
 func makeCaa(rec *models.RecordConfig, ttlop string) string {
