@@ -6,6 +6,7 @@ import (
 	"errors"
 	"flag"
 	"fmt"
+	"maps"
 	"os"
 	"slices"
 	"strings"
@@ -18,7 +19,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/rtypecontrol"
 	"github.com/StackExchange/dnscontrol/v4/pkg/zonerecs"
-	"github.com/miekg/dns/dnsutil"
+	dnsutilv1 "github.com/miekg/dns/dnsutil"
 )
 
 var (
@@ -33,19 +34,30 @@ var globalDCN *domaintags.DomainNameVarieties
 
 // Helper constants/funcs for the CLOUDFLARE proxy testing:
 
-// A-record proxy off/on
+// A-record proxy off/on.
 func CfProxyOff() *TestCase { return tc("proxyoff", cfProxyA("prxy", "174.136.107.111", "off")) }
 func CfProxyOn() *TestCase  { return tc("proxyon", cfProxyA("prxy", "174.136.107.111", "on")) }
 
-// CNAME-record proxy off/on
+// CNAME-record proxy off/on.
 func CfCProxyOff() *TestCase { return tc("cproxyoff", cfProxyCNAME("cproxy", "example.com.", "off")) }
 func CfCProxyOn() *TestCase  { return tc("cproxyon", cfProxyCNAME("cproxy", "example.com.", "on")) }
 
+// Helper constants/funcs for the CLOUDFLARE CNAME flattening testing:
+
+// CNAME flattening off/on (requires paid plan).
+func CfFlattenOff() *TestCase {
+	return tc("flattenoff", cfFlattenCNAME("cflatten", "example.com.", "off"))
+}
+func CfFlattenOn() *TestCase {
+	return tc("flattenon", cfFlattenCNAME("cflatten", "example.com.", "on"))
+}
+
 func getDomainConfigWithNameservers(t *testing.T, prv providers.DNSServiceProvider, domainName string) *models.DomainConfig {
 	dc := &models.DomainConfig{
 		Name: domainName,
 	}
 	dc.PostProcess()
+	rtypecontrol.FixLegacyDC(dc)
 
 	// fix up nameservers
 	ns, err := prv.GetNameservers(domainName)
@@ -105,11 +117,20 @@ func testPermitted(p string, f TestGroup) error {
 }
 
 // makeChanges runs one set of DNS record tests. Returns true on success.
-func makeChanges(t *testing.T, prv providers.DNSServiceProvider, dc *models.DomainConfig, tst *TestCase, desc string, expectChanges bool, origConfig map[string]string) bool {
+func makeChanges(t *testing.T, prv providers.DNSServiceProvider, dc *models.DomainConfig, tst *TestCase, desc string, expectChanges bool, origConfig map[string]string, domainMeta map[string]string) bool {
 	domainName := dc.Name
 
 	return t.Run(desc+":"+tst.Desc, func(t *testing.T) {
 		dom, _ := dc.Copy()
+
+		// Apply domain-level metadata if provided (e.g., for Cloudflare comments/tags management)
+		if domainMeta != nil {
+			if dom.Metadata == nil {
+				dom.Metadata = make(map[string]string)
+			}
+			maps.Copy(dom.Metadata, domainMeta)
+		}
+
 		for _, r := range tst.Records {
 			rc := models.RecordConfig(*r)
 
@@ -144,6 +165,7 @@ func makeChanges(t *testing.T, prv providers.DNSServiceProvider, dc *models.Doma
 			TargetPattern: "",
 		})
 		models.PostProcessRecords(dom.Records)
+		rtypecontrol.FixLegacyDC(dom)
 		dom2, _ := dom.Copy()
 
 		if err := providers.AuditRecords(*providerFlag, dom.Records); err != nil {
@@ -228,12 +250,12 @@ func runTests(t *testing.T, prv providers.DNSServiceProvider, domainName string,
 		// fmt.Printf("DEBUG testPermitted: prov=%q profile=%q\n", *providerFlag, *profileFlag)
 		if err := testPermitted(*profileFlag, *group); err != nil {
 			// t.Logf("%s: ***SKIPPED(%v)***", group.Desc, err)
-			makeChanges(t, prv, dc, tc("Empty"), fmt.Sprintf("%02d:%s ***SKIPPED(%v)***", gIdx, group.Desc, err), false, origConfig)
+			makeChanges(t, prv, dc, tc("Empty"), fmt.Sprintf("%02d:%s ***SKIPPED(%v)***", gIdx, group.Desc, err), false, origConfig, nil)
 			continue
 		}
 
 		// Start the testgroup with a clean slate.
-		makeChanges(t, prv, dc, tc("Empty"), "Clean Slate", false, nil)
+		makeChanges(t, prv, dc, tc("Empty"), "Clean Slate", false, nil, nil)
 
 		// Run the tests.
 		start := time.Now()
@@ -247,7 +269,7 @@ func runTests(t *testing.T, prv providers.DNSServiceProvider, domainName string,
 			//      if t.Failed() {
 			//        break
 			//      }
-			if ok := makeChanges(t, prv, dc, tst, fmt.Sprintf("%02d:%s", gIdx, group.Desc), true, origConfig); !ok {
+			if ok := makeChanges(t, prv, dc, tst, fmt.Sprintf("%02d:%s", gIdx, group.Desc), true, origConfig, group.domainMeta); !ok {
 				break
 			}
 		}
@@ -260,12 +282,13 @@ func runTests(t *testing.T, prv providers.DNSServiceProvider, domainName string,
 }
 
 type TestGroup struct {
-	Desc      string
-	required  []providers.Capability
-	only      []string
-	not       []string
-	trueflags []bool
-	tests     []*TestCase
+	Desc       string
+	required   []providers.Capability
+	only       []string
+	not        []string
+	trueflags  []bool
+	domainMeta map[string]string
+	tests      []*TestCase
 }
 
 type TestCase struct {
@@ -282,7 +305,7 @@ func (tc *TestCase) ExpectNoChanges() *TestCase {
 	return tc
 }
 
-// UnsafeIgnore is the equivalent of DISABLE_IGNORE_SAFETY_CHECK
+// UnsafeIgnore is the equivalent of DISABLE_IGNORE_SAFETY_CHECK.
 func (tc *TestCase) UnsafeIgnore() *TestCase {
 	tc.UnmanagedUnsafe = true
 	return tc
@@ -290,7 +313,7 @@ func (tc *TestCase) UnsafeIgnore() *TestCase {
 
 func SetLabel(r *models.RecordConfig, label, domain string) {
 	r.Name = label
-	r.NameFQDN = dnsutil.AddOrigin(label, "**current-domain**.")
+	r.NameFQDN = dnsutilv1.AddOrigin(label, "**current-domain**.")
 }
 
 func withMeta(record *models.RecordConfig, metadata map[string]string) *models.RecordConfig {
@@ -338,6 +361,27 @@ func cfProxyCNAME(name, target, status string) *models.RecordConfig {
 	return r
 }
 
+func cfFlattenCNAME(name, target, status string) *models.RecordConfig {
+	r := cname(name, target)
+	r.Metadata = make(map[string]string)
+	r.Metadata["cloudflare_cname_flatten"] = status
+	return r
+}
+
+func cfCommentA(name, target, comment string) *models.RecordConfig {
+	r := a(name, target)
+	r.Metadata = make(map[string]string)
+	r.Metadata["cloudflare_comment"] = comment
+	return r
+}
+
+func cfTagsA(name, target, tags string) *models.RecordConfig {
+	r := a(name, target)
+	r.Metadata = make(map[string]string)
+	r.Metadata["cloudflare_tags"] = tags
+	return r
+}
+
 func cfSingleRedirectEnabled() bool {
 	return (*enableCFRedirectMode)
 }
@@ -359,6 +403,10 @@ func cfWorkerRoute(pattern, target string) *models.RecordConfig {
 	return r
 }
 
+func bunnyPullZone(name, pullZoneID string) *models.RecordConfig {
+	return makeRec(name, pullZoneID, "BUNNY_DNS_PZ")
+}
+
 func cfRedir(pattern, target string) *models.RecordConfig {
 	rec, err := rtypecontrol.NewRecordConfigFromRaw(rtypecontrol.FromRawOpts{
 		Type: "CF_REDIRECT",
@@ -404,9 +452,14 @@ func dname(name, target string) *models.RecordConfig {
 }
 
 func ds(name string, keyTag uint16, algorithm, digestType uint8, digest string) *models.RecordConfig {
-	r := makeRec(name, "", "DS")
-	panicOnErr(r.SetTargetDS(keyTag, algorithm, digestType, digest))
-	return r
+	rec, err := rtypecontrol.NewRecordConfigFromRaw(rtypecontrol.FromRawOpts{
+		Type: "DS",
+		TTL:  300,
+		Args: []any{name, keyTag, algorithm, digestType, digest},
+		DCN:  globalDCN,
+	})
+	panicOnErr(err)
+	return rec
 }
 
 func dnskey(name string, flags uint16, protocol, algorithm uint8, publicKey string) *models.RecordConfig {
@@ -591,6 +644,15 @@ func testgroup(desc string, items ...any) *TestGroup {
 				os.Exit(1)
 			}
 			group.trueflags = append(group.trueflags, v.flags...)
+		case domainMetaFilter:
+			if len(group.tests) != 0 {
+				fmt.Printf("ERROR: domainMeta() must be before all tc(): %v\n", desc)
+				os.Exit(1)
+			}
+			if group.domainMeta == nil {
+				group.domainMeta = make(map[string]string)
+			}
+			maps.Copy(group.domainMeta, v.meta)
 		case *TestCase:
 			group.tests = append(group.tests, v)
 		default:
@@ -632,7 +694,7 @@ func txt(name, target string) *models.RecordConfig {
 	return r
 }
 
-// func (r *models.RecordConfig) ttl(t uint32) *models.RecordConfig {
+// func (r *models.RecordConfig) ttl(t uint32) *models.RecordConfig {.
 func ttl(r *models.RecordConfig, t uint32) *models.RecordConfig {
 	r.TTL = t
 	return r
@@ -700,3 +762,11 @@ type alltrueFilter struct {
 func alltrue(f ...bool) alltrueFilter {
 	return alltrueFilter{flags: f}
 }
+
+type domainMetaFilter struct {
+	meta map[string]string
+}
+
+func domainMeta(m map[string]string) domainMetaFilter {
+	return domainMetaFilter{meta: m}
+}
