@@ -127,7 +127,7 @@ func (dc *DomainConfig) Filter(f func(r *RecordConfig) bool) {
 // It will encode:
 // - Name
 // - NameFQDN
-// - Target (CNAME and MX only)
+// - Target (CNAME and MX only).
 func (dc *DomainConfig) Punycode() error {
 	for _, rec := range dc.Records {
 		if rec.IsModernType() {
@@ -152,7 +152,7 @@ func (dc *DomainConfig) Punycode() error {
 			if err := rec.SetTarget(t); err != nil {
 				return err
 			}
-		case "CLOUDFLAREAPI_SINGLE_REDIRECT", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "ADGUARDHOME_A_PASSTHROUGH", "ADGUARDHOME_AAAA_PASSTHROUGH":
+		case "CLOUDFLAREAPI_SINGLE_REDIRECT", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "ADGUARDHOME_A_PASSTHROUGH", "ADGUARDHOME_AAAA_PASSTHROUGH", "BUNNY_DNS_PZ":
 			if err := rec.SetTarget(rec.GetTargetField()); err != nil {
 				return err
 			}
