@@ -1,6 +1,6 @@
 package diff2
 
-//go:generate stringer -type=Verb
+//go:generate go tool stringer -type=Verb
 
 // This module provides functions that "diff" the existing records
 // against the desired records.
@@ -25,7 +25,7 @@ const (
 	REPORT             // No change, but I have something to say!
 )
 
-// ChangeList is a list of Change
+// ChangeList is a list of Change.
 type ChangeList []Change
 
 // Change is an instruction to the provider. Generally if one properly executes
@@ -115,7 +115,7 @@ General instructions:
 */
 
 // CreateCorrection creates a new Correction based on the given
-// function and prefills it with the Msg of the current Change
+// function and prefills it with the Msg of the current Change.
 func (c *Change) CreateCorrection(correctionFunction func() error) *models.Correction {
 	return &models.Correction{
 		F:   correctionFunction,
@@ -124,7 +124,7 @@ func (c *Change) CreateCorrection(correctionFunction func() error) *models.Corre
 }
 
 // CreateMessage creates a new correction with only the message.
-// Used for diff2.Report corrections
+// Used for diff2.Report corrections.
 func (c *Change) CreateMessage() *models.Correction {
 	return &models.Correction{
 		Msg: c.MsgsJoined,
@@ -133,7 +133,7 @@ func (c *Change) CreateMessage() *models.Correction {
 
 // CreateCorrectionWithMessage creates a new Correction based on the
 // given function and prefixes given function with the Msg of the
-// current change
+// current change.
 func (c *Change) CreateCorrectionWithMessage(msg string, correctionFunction func() error) *models.Correction {
 	return &models.Correction{
 		F:   correctionFunction,
@@ -150,8 +150,8 @@ func (c *Change) CreateCorrectionWithMessage(msg string, correctionFunction func
 // record, if A records are added, changed, or removed, the API takes
 // www.example.com, A, and a list of all the desired IP addresses.
 //
-// Examples include: AZURE_DNS, GCORE, NS1, ROUTE53
-func ByRecordSet(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, error) {
+// Examples include: AZURE_DNS, GCORE, NS1, ROUTE53.
+func ByRecordSet(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, int, error) {
 	return byHelper(analyzeByRecordSet, existing, dc, compFunc)
 }
 
@@ -162,8 +162,8 @@ func ByRecordSet(existing models.Records, dc *models.DomainConfig, compFunc Comp
 // time. That is, updates are done by sending a list of DNS records
 // to be served at a particular label, or the label itself is deleted.
 //
-// Examples include: GANDI_V5
-func ByLabel(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, error) {
+// Examples include: GANDI_V5.
+func ByLabel(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, int, error) {
 	return byHelper(analyzeByLabel, existing, dc, compFunc)
 }
 
@@ -178,8 +178,8 @@ func ByLabel(existing models.Records, dc *models.DomainConfig, compFunc Comparab
 // A change always has exactly 1 old and 1 new: .Old[0] and .New[0]
 // A delete always has exactly 1 old: .Old[0]
 //
-// Examples include: CLOUDFLAREAPI, HEDNS, INWX, MSDNS, OVH, PORKBUN, VULTR
-func ByRecord(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, error) {
+// Examples include: CLOUDFLAREAPI, HEDNS, INWX, MSDNS, OVH, PORKBUN, VULTR.
+func ByRecord(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, int, error) {
 	return byHelper(analyzeByRecord, existing, dc, compFunc)
 }
 
@@ -195,51 +195,64 @@ func ByRecord(existing models.Records, dc *models.DomainConfig, compFunc Compara
 //
 // Example usage:
 //
-//	msgs, changes, err := diff2.ByZone(foundRecords, dc, nil)
+//	result, err := diff2.ByZone(foundRecords, dc, nil)
 //	if err != nil {
 //	  return nil, err
 //	}
 //	if changes {
 //		// Generate a "correction" that uploads the entire zone.
-//		// (dc.Records are the new records for the zone).
+//		// (result.DesiredPlus are the new records for the zone).
 //	}
 //
-// Example providers include: BIND, AUTODNS
-func ByZone(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) ([]string, bool, error) {
-	// Only return the messages.  The caller has the list of records needed to build the new zone.
-	instructions, err := byHelper(analyzeByRecord, existing, dc, compFunc)
-	changes := false
-	for i := range instructions {
-		//fmt.Printf("DEBUG: ByZone #%d: %v\n", i, ii)
-		if instructions[i].Type != REPORT {
-			changes = true
-		}
-	}
-	return justMsgs(instructions), changes, err
+// Example providers include: BIND, AUTODNS.
+func ByZone(existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ByResults, error) {
+	// Only return the messages and a list of records needed to build the new zone.
+	result, err := byHelperStruct(analyzeByRecord, existing, dc, compFunc)
+	result.Msgs = justMsgs(result.Instructions)
+	return result, err
 }
 
-//
+// ByResults is the results of ByZone() and perhaps someday all the By*() functions.
+// It is partially populated by // byHelperStruct() and partially by the By*()
+// functions that use it.
+type ByResults struct {
+	// Fields filled in by byHelperStruct():
+	Instructions      ChangeList     // Instructions to turn existing into desired.
+	ActualChangeCount int            // Number of actual changes, not including REPORTs.
+	HasChanges        bool           // True if there are any changes.
+	DesiredPlus       models.Records // Desired + foreign + ignored
+	// Fields filled in by ByZone():
+	Msgs []string // Just the messages from the instructions.
+}
 
-// byHelper does 90% of the work for the By*() calls.
-func byHelper(fn func(cc *CompareConfig) ChangeList, existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, error) {
+// byHelper is like byHelperStruct but has a signature that is compatible with legacy code.
+// Deprecated: Use byHelperStruct instead.
+func byHelper(fn func(cc *CompareConfig) (ChangeList, int), existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ChangeList, int, error) {
+	result, err := byHelperStruct(fn, existing, dc, compFunc)
+	return result.Instructions, result.ActualChangeCount, err
+}
 
+// byHelperStruct does 90% of the work for the By*() calls.
+func byHelperStruct(fn func(cc *CompareConfig) (ChangeList, int), existing models.Records, dc *models.DomainConfig, compFunc ComparableFunc) (ByResults, error) {
 	// Process NO_PURGE/ENSURE_ABSENT and IGNORE*().
-	desired, msgs, err := handsoff(
+	desiredPlus, msgs, err := handsoff(
 		dc.Name,
 		existing, dc.Records, dc.EnsureAbsent,
 		dc.Unmanaged,
 		dc.UnmanagedUnsafe,
 		dc.KeepUnknown,
+		dc.IgnoreExternalDNS,
+		dc.ExternalDNSPrefix,
 	)
 	if err != nil {
-		return nil, err
+		return ByResults{}, err
 	}
 
 	// Regroup existing/desiredd for easy comparison:
-	cc := NewCompareConfig(dc.Name, existing, desired, compFunc)
+	cc := NewCompareConfig(dc.Name, existing, desiredPlus, compFunc)
 
 	// Analyze and generate the instructions:
-	instructions := fn(cc)
+	instructions, actualChangeCount := fn(cc)
 
 	// If we have msgs, create a change to output them:
 	if len(msgs) != 0 {
@@ -252,5 +265,10 @@ func byHelper(fn func(cc *CompareConfig) ChangeList, existing models.Records, dc
 		instructions = append([]Change{chg}, instructions...)
 	}
 
-	return instructions, nil
+	return ByResults{
+		Instructions:      instructions,
+		ActualChangeCount: actualChangeCount,
+		HasChanges:        actualChangeCount > 0,
+		DesiredPlus:       desiredPlus,
+	}, nil
 }
