@@ -19,12 +19,12 @@ func init() {
 	})
 }
 
-// slackNotifier sends notifications to slack or mattermost
+// slackNotifier sends notifications to slack or mattermost.
 type slackNotifier struct {
 	URL string
 }
 
-func (s *slackNotifier) Notify(domain, provider, msg string, err error, preview bool) {
+func (s *slackNotifier) Notify(domain, provider, msg string, err error, preview bool) error {
 	var payload struct {
 		Username string `json:"username"`
 		Text     string `json:"text"`
@@ -40,7 +40,8 @@ func (s *slackNotifier) Notify(domain, provider, msg string, err error, preview
 	}
 
 	json, _ := json.Marshal(payload)
-	http.Post(s.URL, "text/json", bytes.NewReader(json))
+	_, posterr := http.Post(s.URL, "text/json", bytes.NewReader(json))
+	return posterr
 }
 
 func (s *slackNotifier) Done() {}
