@@ -13,7 +13,7 @@ import (
 // CLI is an abstraction around the CLI.
 type CLI interface {
 	Printer
-	StartDomain(domain string)
+	StartDomain(dc *models.DomainConfig)
 	StartDNSProvider(name string, skip bool)
 	EndProvider(name string, numCorrections int, err error)
 	EndProvider2(name string, numCorrections int)
@@ -27,21 +27,21 @@ type CLI interface {
 
 // Printer is a simple abstraction for printing data. Can be passed to providers to give simple output capabilities.
 type Printer interface {
-	Debugf(fmt string, args ...interface{})
-	Printf(fmt string, args ...interface{})
+	Debugf(fmt string, args ...any)
+	Printf(fmt string, args ...any)
 	Println(lines ...string)
-	Warnf(fmt string, args ...interface{})
-	Errorf(fmt string, args ...interface{})
-	PrintfIf(print bool, fmt string, args ...interface{})
+	Warnf(fmt string, args ...any)
+	Errorf(fmt string, args ...any)
+	PrintfIf(prnt bool, fmt string, args ...any)
 }
 
 // Debugf is called to print/format debug information.
-func Debugf(fmt string, args ...interface{}) {
+func Debugf(fmt string, args ...any) {
 	DefaultPrinter.Debugf(fmt, args...)
 }
 
 // Printf is called to print/format information.
-func Printf(fmt string, args ...interface{}) {
+func Printf(fmt string, args ...any) {
 	DefaultPrinter.Printf(fmt, args...)
 }
 
@@ -51,35 +51,33 @@ func Println(lines ...string) {
 }
 
 // Warnf is called to print/format a warning.
-func Warnf(fmt string, args ...interface{}) {
+func Warnf(fmt string, args ...any) {
 	DefaultPrinter.Warnf(fmt, args...)
 }
 
 // Errorf is called to print/format an error.
-// func Errorf(fmt string, args ...interface{}) {
+// func Errorf(fmt string, args ...any) {
 // 	DefaultPrinter.Errorf(fmt, args...)
 // }
 
 // PrintfIf is called to optionally print something.
-func PrintfIf(print bool, fmt string, args ...interface{}) {
-	DefaultPrinter.PrintfIf(print, fmt, args...)
+func PrintfIf(prnt bool, fmt string, args ...any) {
+	DefaultPrinter.PrintfIf(prnt, fmt, args...)
 }
 
-var (
-	// DefaultPrinter is the default Printer, used by Debugf, Printf, and Warnf.
-	DefaultPrinter = &ConsolePrinter{
-		Reader:  bufio.NewReader(os.Stdin),
-		Writer:  os.Stdout,
-		Verbose: false,
-	}
-)
+// DefaultPrinter is the default Printer, used by Debugf, Printf, and Warnf.
+var DefaultPrinter = &ConsolePrinter{
+	Reader:  bufio.NewReader(os.Stdin),
+	Writer:  os.Stdout,
+	Verbose: false,
+}
 
-// SkinnyReport is true to to disable certain print statements.
+// SkinnyReport is true to disable certain print statements.
 // This is a hack until we have the new printer replacement. The long
 // variable name is easy to grep for when we make the conversion.
 var SkinnyReport = true
 
-// MaxReport represents how many records to show if SkinnyReport == true
+// MaxReport represents how many records to show if SkinnyReport == true.
 var MaxReport = 5
 
 // ConsolePrinter is a handle for the console printer.
@@ -91,8 +89,8 @@ type ConsolePrinter struct {
 }
 
 // StartDomain is called at the start of each domain.
-func (c ConsolePrinter) StartDomain(domain string) {
-	fmt.Fprintf(c.Writer, "******************** Domain: %s\n", domain)
+func (c ConsolePrinter) StartDomain(dc *models.DomainConfig) {
+	fmt.Fprintf(c.Writer, "******************** Domain: %s\n", dc.DisplayName)
 }
 
 // PrintCorrection is called to print/format each correction.
@@ -109,10 +107,7 @@ func (c ConsolePrinter) PrintReport(i int, correction *models.Correction) {
 func (c ConsolePrinter) PromptToRun() bool {
 	fmt.Fprint(c.Writer, "Run? (y/N): ")
 	txt, err := c.Reader.ReadString('\n')
-	run := true
-	if err != nil {
-		run = false
-	}
+	run := err == nil
 	txt = strings.ToLower(strings.TrimSpace(txt))
 	if txt != "y" {
 		run = false
@@ -184,14 +179,14 @@ func (c ConsolePrinter) EndProvider2(name string, numCorrections int) {
 }
 
 // Debugf is called to print/format debug information.
-func (c ConsolePrinter) Debugf(format string, args ...interface{}) {
+func (c ConsolePrinter) Debugf(format string, args ...any) {
 	if c.Verbose {
 		fmt.Fprintf(c.Writer, format, args...)
 	}
 }
 
 // Printf is called to print/format information.
-func (c ConsolePrinter) Printf(format string, args ...interface{}) {
+func (c ConsolePrinter) Printf(format string, args ...any) {
 	fmt.Fprintf(c.Writer, format, args...)
 }
 
@@ -201,18 +196,18 @@ func (c ConsolePrinter) Println(lines ...string) {
 }
 
 // Warnf is called to print/format a warning.
-func (c ConsolePrinter) Warnf(format string, args ...interface{}) {
+func (c ConsolePrinter) Warnf(format string, args ...any) {
 	fmt.Fprintf(c.Writer, "WARNING: "+format, args...)
 }
 
 // Errorf is called to print/format an error.
-func (c ConsolePrinter) Errorf(format string, args ...interface{}) {
+func (c ConsolePrinter) Errorf(format string, args ...any) {
 	fmt.Fprintf(c.Writer, "ERROR: "+format, args...)
 }
 
 // PrintfIf is called to optionally print/format a message.
-func (c ConsolePrinter) PrintfIf(print bool, format string, args ...interface{}) {
-	if print {
+func (c ConsolePrinter) PrintfIf(prnt bool, format string, args ...any) {
+	if prnt {
 		fmt.Fprintf(c.Writer, format, args...)
 	}
 }
