@@ -1,6 +1,7 @@
 package spflib
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 )
@@ -10,6 +11,21 @@ type SPFRecord struct {
 	Parts []*SPFPart
 }
 
+// Lookups returns the number of DNS lookups required by s.
+// NB(tlim): This needs to be in this file or "gopherjs build" fails.
+func (s *SPFRecord) Lookups() int {
+	count := 0
+	for _, p := range s.Parts {
+		if p.IsLookup {
+			count++
+		}
+		if p.IncludeRecord != nil {
+			count += p.IncludeRecord.Lookups()
+		}
+	}
+	return count
+}
+
 // SPFPart stores a part of an SPF record, with attributes.
 type SPFPart struct {
 	Text          string
@@ -28,7 +44,7 @@ var qualifiers = map[byte]bool{
 // Parse parses a raw SPF record.
 func Parse(text string, dnsres Resolver) (*SPFRecord, error) {
 	if !strings.HasPrefix(text, "v=spf1 ") {
-		return nil, fmt.Errorf("not an SPF record")
+		return nil, errors.New("not an SPF record")
 	}
 	parts := strings.Split(text, " ")
 	rec := &SPFRecord{}
@@ -40,13 +56,14 @@ func Parse(text string, dnsres Resolver) (*SPFRecord, error) {
 		if qualifiers[part[0]] {
 			part = part[1:]
 		}
+		lcpart := strings.ToLower(part) // We have seen "Ip4" instead of "ip4".  Let's be gracious and allow it.
 		rec.Parts = append(rec.Parts, p)
 		if part == "all" {
 			// all. nothing else matters.
 			break
-		} else if strings.HasPrefix(part, "a") || strings.HasPrefix(part, "mx") {
+		} else if strings.HasPrefix(lcpart, "a") || strings.HasPrefix(lcpart, "mx") {
 			p.IsLookup = true
-		} else if strings.HasPrefix(part, "ip4:") || strings.HasPrefix(part, "ip6:") {
+		} else if strings.HasPrefix(lcpart, "ip4:") || strings.HasPrefix(lcpart, "ip6:") {
 			// ip address, 0 lookups
 			continue
 		} else if strings.HasPrefix(part, "include:") || strings.HasPrefix(part, "redirect=") {
@@ -73,7 +90,7 @@ func Parse(text string, dnsres Resolver) (*SPFRecord, error) {
 				}
 				p.IncludeRecord, err = Parse(subRecord, dnsres)
 				if err != nil {
-					return nil, fmt.Errorf("in included SPF: %s", err)
+					return nil, fmt.Errorf("in included SPF: %w", err)
 				}
 			}
 		} else if strings.HasPrefix(part, "exists:") || strings.HasPrefix(part, "ptr:") {
@@ -81,7 +98,6 @@ func Parse(text string, dnsres Resolver) (*SPFRecord, error) {
 		} else {
 			return nil, fmt.Errorf("unsupported SPF part %s", part)
 		}
-
 	}
 	return rec, nil
 }
