@@ -24,28 +24,33 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/bindserial"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
+	"github.com/StackExchange/dnscontrol/v4/pkg/dnsrr"
+	"github.com/StackExchange/dnscontrol/v4/pkg/domaintags"
 	"github.com/StackExchange/dnscontrol/v4/pkg/prettyzone"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
-	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/miekg/dns"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/rtypecontrol"
+	"github.com/StackExchange/dnscontrol/v4/pkg/rtypeinfo"
+	dnsv1 "github.com/miekg/dns"
 )
 
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can("Just writes out a comment indicating DNSSEC was requested"),
-	providers.CanGetZones:            providers.Can(),
 	providers.CanConcur:              providers.Can(),
+	providers.CanGetZones:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDHCID:            providers.Can(),
 	providers.CanUseDNAME:            providers.Can(),
-	providers.CanUseDS:               providers.Can(),
 	providers.CanUseDNSKEY:           providers.Can(),
+	providers.CanUseDS:               providers.Can(),
 	providers.CanUseHTTPS:            providers.Can(),
 	providers.CanUseLOC:              providers.Can(),
 	providers.CanUseNAPTR:            providers.Can(),
 	providers.CanUseOPENPGPKEY:       providers.Can(),
 	providers.CanUsePTR:              providers.Can(),
+	providers.CanUseRP:               providers.Can(),
 	providers.CanUseSMIMEA:           providers.Can(),
 	providers.CanUseSOA:              providers.Can(),
 	providers.CanUseSRV:              providers.Can(),
@@ -68,7 +73,7 @@ func initBind(config map[string]string, providermeta json.RawMessage) (providers
 		api.directory = "zones"
 	}
 	if api.filenameformat == "" {
-		api.filenameformat = "%U.zone"
+		api.filenameformat = "%c.zone"
 	}
 	if len(providermeta) != 0 {
 		err := json.Unmarshal(providermeta, api)
@@ -139,7 +144,7 @@ func (c *bindProvider) GetNameservers(string) ([]*models.Nameserver, error) {
 	return models.ToNameservers(r)
 }
 
-// ListZones returns all the zones in an account
+// ListZones returns all the zones in an account.
 func (c *bindProvider) ListZones() ([]string, error) {
 	if _, err := os.Stat(c.directory); os.IsNotExist(err) {
 		return nil, fmt.Errorf("directory %q does not exist", c.directory)
@@ -167,24 +172,30 @@ func (c *bindProvider) GetZoneRecords(domain string, meta map[string]string) (mo
 	if _, err := os.Stat(c.directory); os.IsNotExist(err) {
 		printer.Printf("\nWARNING: BIND directory %q does not exist! (will create)\n", c.directory)
 	}
-	_, okTag := meta[models.DomainTag]
-	_, okUnique := meta[models.DomainUniqueName]
-	if !okTag && !okUnique {
-		// This layering violation is needed for tests only.
-		// Otherwise, this is set already.
-		// Note: In this situation there is no "uniquename" or "tag".
-		zonefile = filepath.Join(c.directory,
-			makeFileName(c.filenameformat, domain, domain, ""))
-	} else {
-		zonefile = filepath.Join(c.directory,
-			makeFileName(c.filenameformat,
-				meta[models.DomainUniqueName], domain, meta[models.DomainTag]),
-		)
+	ff := domaintags.DomainNameVarieties{
+		Tag:         meta[models.DomainTag],
+		NameRaw:     meta[models.DomainNameRaw],
+		NameASCII:   domain,
+		NameUnicode: meta[models.DomainNameUnicode],
+		UniqueName:  meta[models.DomainUniqueName],
+		// NB(tlim): When "get-zones" is called, these values are populated
+		// directly by commands/getZones.go near where provider.GetZoneRecords()
+		// is called. Changes here may need to be reflected there too.
 	}
+	zonefile = filepath.Join(c.directory,
+		makeFileName(
+			c.filenameformat,
+			ff,
+		),
+	)
+	//fmt.Printf("DEBUG: Reading zonefile %q\n", zonefile)
+	//fmt.Printf("DEBUG: Meta %+v\n", meta)
+	//fmt.Printf("DEBUG: Domain Names %+v\n", ff)
+
 	content, err := os.ReadFile(zonefile)
 	if os.IsNotExist(err) {
 		// If the file doesn't exist, that's not an error. Just informational.
-		fmt.Fprintf(os.Stderr, "File does not yet exist: %q (will create)\n", zonefile)
+		fmt.Fprintf(os.Stderr, "INFO: File does not (yet) exist: %q\n", zonefile)
 		return nil, nil
 	}
 	if err != nil {
@@ -196,14 +207,33 @@ func (c *bindProvider) GetZoneRecords(domain string, meta map[string]string) (mo
 
 // ParseZoneContents parses a string as a BIND zone and returns the records.
 func ParseZoneContents(content string, zoneName string, zonefileName string) (models.Records, error) {
-	zp := dns.NewZoneParser(strings.NewReader(content), zoneName, zonefileName)
+	zp := dnsv1.NewZoneParser(strings.NewReader(content), zoneName, zonefileName)
 
 	foundRecords := models.Records{}
 	for rr, ok := zp.Next(); ok; rr, ok = zp.Next() {
-		rec, err := models.RRtoRCTxtBug(rr, zoneName)
-		if err != nil {
-			return nil, err
+		var rec models.RecordConfig
+		var prec *models.RecordConfig
+		var err error
+
+		rtype := rr.Header().Rrtype
+		rtypeStr := dnsv1.TypeToString[rtype]
+		if rtypeinfo.IsModernType(rtypeStr) {
+			// Modern types:
+			name := rr.Header().Name
+			prec, err = rtypecontrol.NewRecordConfigFromStruct(name, rr.Header().Ttl, rtypeStr, rr, domaintags.MakeDomainNameVarieties(zoneName))
+			if err != nil {
+				return nil, err
+			}
+			rec = *prec
+			rec.TTL = rr.Header().Ttl
+		} else {
+			// Legacy types:
+			rec, err = dnsrr.RRtoRCTxtBug(rr, zoneName)
+			if err != nil {
+				return nil, err
+			}
 		}
+
 		foundRecords = append(foundRecords, &rec)
 	}
 
@@ -213,7 +243,7 @@ func ParseZoneContents(content string, zoneName string, zonefileName string) (mo
 	return foundRecords, nil
 }
 
-func (c *bindProvider) EnsureZoneExists(_ string) error {
+func (c *bindProvider) EnsureZoneExists(_ string, _ map[string]string) error {
 	return nil
 }
 
@@ -273,8 +303,16 @@ func (c *bindProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, foundR
 	}
 
 	zonefile = filepath.Join(c.directory,
-		makeFileName(c.filenameformat,
-			dc.Metadata[models.DomainUniqueName], dc.Name, dc.Metadata[models.DomainTag]),
+		makeFileName(
+			c.filenameformat,
+			domaintags.DomainNameVarieties{
+				Tag:         dc.Tag,
+				NameRaw:     dc.NameRaw,
+				NameASCII:   dc.Name,
+				NameUnicode: dc.NameUnicode,
+				UniqueName:  dc.UniqueName,
+			},
+		),
 	)
 
 	// We only change the serial number if there is a change.
