@@ -6,46 +6,38 @@ import (
 	"fmt"
 	"strings"
 
-	"golang.org/x/net/idna"
-
 	"github.com/StackExchange/dnscontrol/v4/models"
+	"github.com/StackExchange/dnscontrol/v4/pkg/domaintags"
+	"github.com/StackExchange/dnscontrol/v4/pkg/rtypecontrol"
+	"github.com/StackExchange/dnscontrol/v4/pkg/txtutil"
+	"github.com/StackExchange/dnscontrol/v4/providers/cloudflare/rtypes/cfsingleredirect"
 	"github.com/cloudflare/cloudflare-go"
+	"golang.org/x/net/idna"
 )
 
-// get list of domains for account. Cache so the ids can be looked up from domain name
-// The caller must do all locking.
-func (c *cloudflareProvider) cacheDomainList() error {
-	if c.domainIndex != nil {
-		return nil
-	}
-
-	c.domainIndex = map[string]string{}
-	c.nameservers = map[string][]string{}
-	//fmt.Printf("DEBUG: CLOUDFLARE POPULATING CACHE\n")
+func (c *cloudflareProvider) fetchAllZones() (map[string]cloudflare.Zone, error) {
 	zones, err := c.cfClient.ListZones(context.Background())
 	if err != nil {
-		return fmt.Errorf("failed fetching domain list from cloudflare(%q): %s", c.cfClient.APIEmail, err)
+		return nil, fmt.Errorf("failed fetching domain list from cloudflare(%q): %w", c.cfClient.APIEmail, err)
 	}
 
+	m := make(map[string]cloudflare.Zone, len(zones))
 	for _, zone := range zones {
 		if encoded, err := idna.ToASCII(zone.Name); err == nil && encoded != zone.Name {
-			if _, ok := c.domainIndex[encoded]; ok {
+			if _, ok := m[encoded]; ok {
 				fmt.Printf("WARNING: Zone %q appears twice in this cloudflare account\n", encoded)
 			}
-			c.domainIndex[encoded] = zone.ID
-			c.nameservers[encoded] = zone.NameServers
+			m[encoded] = zone
 		}
-		if _, ok := c.domainIndex[zone.Name]; ok {
+		if _, ok := m[zone.Name]; ok {
 			fmt.Printf("WARNING: Zone %q appears twice in this cloudflare account\n", zone.Name)
 		}
-		c.domainIndex[zone.Name] = zone.ID
-		c.nameservers[zone.Name] = zone.NameServers
+		m[zone.Name] = zone
 	}
-
-	return nil
+	return m, nil
 }
 
-// get all records for a domain
+// get all records for a domain.
 func (c *cloudflareProvider) getRecordsForDomain(id string, domain string) ([]*models.RecordConfig, error) {
 	records := []*models.RecordConfig{}
 	rrs, _, err := c.cfClient.ListDNSRecords(context.Background(), cloudflare.ZoneIdentifier(id), cloudflare.ListDNSRecordsParams{})
@@ -57,7 +49,11 @@ func (c *cloudflareProvider) getRecordsForDomain(id string, domain string) ([]*m
 		if err != nil {
 			return nil, err
 		}
-		records = append(records, rt)
+		// nativeToRecord may return nil if the record is supposed to be skipped
+		// i.e. read only, cloudflare-managed, etc.
+		if rt != nil {
+			records = append(records, rt)
+		}
 	}
 	return records, nil
 }
@@ -68,7 +64,14 @@ func (c *cloudflareProvider) deleteDNSRecord(rec cloudflare.DNSRecord, domainID
 
 func (c *cloudflareProvider) createZone(domainName string) (string, error) {
 	zone, err := c.cfClient.CreateZone(context.Background(), domainName, false, cloudflare.Account{ID: c.accountID}, "full")
-	return zone.ID, err
+	if err != nil {
+		return "", err
+	}
+	if encoded, err := idna.ToASCII(zone.Name); err == nil && encoded != zone.Name {
+		c.zoneCache.SetZone(encoded, zone)
+	}
+	c.zoneCache.SetZone(domainName, zone)
+	return zone.ID, nil
 }
 
 func cfDnskeyData(rec *models.RecordConfig) *cfRecData {
@@ -136,6 +139,26 @@ func cfSvcbData(rec *models.RecordConfig) *cfRecData {
 	}
 }
 
+func cfLocData(rec *models.RecordConfig) *cfRecData {
+	latDir, latDeg, latMin, latSec := models.ReverseLatitude(rec.LocLatitude)
+	longDir, longDeg, longMin, longSec := models.ReverseLongitude(rec.LocLongitude)
+
+	return &cfRecData{
+		Altitude:      models.ReverseAltitude(rec.LocAltitude),
+		LatDegrees:    latDeg,
+		LatDirection:  latDir,
+		LatMinutes:    latMin,
+		LatSeconds:    latSec,
+		LongDegrees:   longDeg,
+		LongDirection: longDir,
+		LongMinutes:   longMin,
+		LongSeconds:   longSec,
+		PrecisionHorz: models.ReverseENotationInt(rec.LocHorizPre),
+		PrecisionVert: models.ReverseENotationInt(rec.LocVertPre),
+		Size:          models.ReverseENotationInt(rec.LocSize),
+	}
+}
+
 func cfNaptrData(rec *models.RecordConfig) *cfNaptrRecData {
 	return &cfNaptrRecData{
 		Flags:       rec.NaptrFlags,
@@ -148,19 +171,17 @@ func cfNaptrData(rec *models.RecordConfig) *cfNaptrRecData {
 }
 
 func (c *cloudflareProvider) createRecDiff2(rec *models.RecordConfig, domainID string, msg string) []*models.Correction {
-
 	content := rec.GetTargetField()
 	if rec.Metadata[metaOriginalIP] != "" {
 		content = rec.Metadata[metaOriginalIP]
 	}
 	prio := ""
-	if rec.Type == "MX" {
+	switch rec.Type {
+	case "MX":
 		prio = fmt.Sprintf(" %d ", rec.MxPreference)
-	}
-	if rec.Type == "TXT" {
-		content = rec.GetTargetTXTJoined()
-	}
-	if rec.Type == "DS" {
+	case "TXT":
+		content = txtutil.EncodeQuoted(rec.GetTargetTXTJoined())
+	case "DS":
 		content = fmt.Sprintf("%d %d %d %s", rec.DsKeyTag, rec.DsAlgorithm, rec.DsDigestType, rec.DsDigest)
 	}
 	if msg == "" {
@@ -169,6 +190,15 @@ func (c *cloudflareProvider) createRecDiff2(rec *models.RecordConfig, domainID s
 	if rec.Metadata[metaProxy] == "on" || rec.Metadata[metaProxy] == "full" {
 		msg = msg + fmt.Sprintf("\nACTIVATE PROXY for new record %s %s %d %s", rec.GetLabel(), rec.Type, rec.TTL, rec.GetTargetField())
 	}
+	if rec.Metadata[metaCNAMEFlatten] == "on" {
+		msg = msg + fmt.Sprintf("\nENABLE CNAME FLATTENING for new record %s %s", rec.GetLabel(), rec.Type)
+	}
+	if rec.Metadata[metaComment] != "" {
+		msg = msg + fmt.Sprintf("\nSET COMMENT for new record %s %s: %q", rec.GetLabel(), rec.Type, rec.Metadata[metaComment])
+	}
+	if rec.Metadata[metaTags] != "" {
+		msg = msg + fmt.Sprintf("\nSET TAGS for new record %s %s: %s", rec.GetLabel(), rec.Type, rec.Metadata[metaTags])
+	}
 	arr := []*models.Correction{{
 		Msg: msg,
 		F: func() error {
@@ -179,28 +209,44 @@ func (c *cloudflareProvider) createRecDiff2(rec *models.RecordConfig, domainID s
 				Content:  content,
 				Priority: &rec.MxPreference,
 			}
-			if rec.Type == "SRV" {
+			// Set comment if specified
+			if comment := rec.Metadata[metaComment]; comment != "" {
+				cf.Comment = comment
+			}
+			// Set tags if specified
+			if tags := rec.Metadata[metaTags]; tags != "" {
+				cf.Tags = strings.Split(tags, ",")
+			}
+			// Set CNAME flattening setting if enabled
+			if rec.Type == "CNAME" && rec.Metadata[metaCNAMEFlatten] == "on" {
+				flatten := true
+				cf.Settings = cloudflare.DNSRecordSettings{FlattenCNAME: &flatten}
+			}
+			switch rec.Type {
+			case "SRV":
 				cf.Data = cfSrvData(rec)
 				cf.Name = rec.GetLabelFQDN()
-			} else if rec.Type == "CAA" {
+			case "CAA":
 				cf.Data = cfCaaData(rec)
 				cf.Name = rec.GetLabelFQDN()
 				cf.Content = ""
-			} else if rec.Type == "TLSA" {
+			case "TLSA":
 				cf.Data = cfTlsaData(rec)
 				cf.Name = rec.GetLabelFQDN()
-			} else if rec.Type == "SSHFP" {
+			case "SSHFP":
 				cf.Data = cfSshfpData(rec)
 				cf.Name = rec.GetLabelFQDN()
-			} else if rec.Type == "DNSKEY" {
+			case "DNSKEY":
 				cf.Data = cfDnskeyData(rec)
-			} else if rec.Type == "DS" {
+			case "DS":
 				cf.Data = cfDSData(rec)
-			} else if rec.Type == "NAPTR" {
+			case "NAPTR":
 				cf.Data = cfNaptrData(rec)
 				cf.Name = rec.GetLabelFQDN()
-			} else if rec.Type == "HTTPS" || rec.Type == "SVCB" {
+			case "HTTPS", "SVCB":
 				cf.Data = cfSvcbData(rec)
+			case "LOC":
+				cf.Data = cfLocData(rec)
 			}
 			resp, err := c.cfClient.CreateDNSRecord(context.Background(), cloudflare.ZoneIdentifier(domainID), cf)
 			if err != nil {
@@ -220,7 +266,7 @@ func (c *cloudflareProvider) createRecDiff2(rec *models.RecordConfig, domainID s
 
 func (c *cloudflareProvider) modifyRecord(domainID, recID string, proxied bool, rec *models.RecordConfig) error {
 	if domainID == "" || recID == "" {
-		return fmt.Errorf("cannot modify record if domain or record id are empty")
+		return errors.New("cannot modify record if domain or record id are empty")
 	}
 
 	r := cloudflare.UpdateDNSRecordParams{
@@ -232,45 +278,67 @@ func (c *cloudflareProvider) modifyRecord(domainID, recID string, proxied bool,
 		Priority: &rec.MxPreference,
 		TTL:      int(rec.TTL),
 	}
-	if rec.Type == "TXT" {
-		r.Content = rec.GetTargetTXTJoined()
+
+	// Handle CNAME flattening setting
+	if rec.Type == "CNAME" {
+		flatten := rec.Metadata[metaCNAMEFlatten] == "on"
+		r.Settings = cloudflare.DNSRecordSettings{FlattenCNAME: &flatten}
 	}
-	if rec.Type == "SRV" {
+
+	// Set comment if specified (nil keeps current, "" empties it, value sets it)
+	if comment, ok := rec.Metadata[metaComment]; ok {
+		r.Comment = &comment
+	}
+	// Set tags if specified (empty key means clear all tags)
+	if tags, ok := rec.Metadata[metaTags]; ok {
+		if tags != "" {
+			r.Tags = strings.Split(tags, ",")
+		} else {
+			r.Tags = []string{}
+		}
+	}
+
+	switch rec.Type {
+	case "TXT":
+		r.Content = txtutil.EncodeQuoted(rec.GetTargetTXTJoined())
+	case "SRV":
 		r.Data = cfSrvData(rec)
 		r.Name = rec.GetLabelFQDN()
-	} else if rec.Type == "CAA" {
+	case "CAA":
 		r.Data = cfCaaData(rec)
 		r.Name = rec.GetLabelFQDN()
 		r.Content = ""
-	} else if rec.Type == "TLSA" {
+	case "TLSA":
 		r.Data = cfTlsaData(rec)
 		r.Name = rec.GetLabelFQDN()
-	} else if rec.Type == "SSHFP" {
+	case "SSHFP":
 		r.Data = cfSshfpData(rec)
 		r.Name = rec.GetLabelFQDN()
-	} else if rec.Type == "DNSKEY" {
+	case "DNSKEY":
 		r.Data = cfDnskeyData(rec)
 		r.Content = ""
-	} else if rec.Type == "DS" {
+	case "DS":
 		r.Data = cfDSData(rec)
 		r.Content = ""
-	} else if rec.Type == "NAPTR" {
+	case "NAPTR":
 		r.Data = cfNaptrData(rec)
 		r.Name = rec.GetLabelFQDN()
-	} else if rec.Type == "HTTPS" || rec.Type == "SVCB" {
+	case "HTTPS", "SVCB":
 		r.Data = cfSvcbData(rec)
+	case "LOC":
+		r.Data = cfLocData(rec)
 	}
 	_, err := c.cfClient.UpdateDNSRecord(context.Background(), cloudflare.ZoneIdentifier(domainID), r)
 	return err
 }
 
-// change universal ssl state
+// change universal ssl state.
 func (c *cloudflareProvider) changeUniversalSSL(domainID string, state bool) error {
 	_, err := c.cfClient.EditUniversalSSLSetting(context.Background(), domainID, cloudflare.UniversalSSLSetting{Enabled: state})
 	return err
 }
 
-// get universal ssl state
+// get universal ssl state.
 func (c *cloudflareProvider) getUniversalSSL(domainID string) (bool, error) {
 	result, err := c.cfClient.UniversalSSLSettingDetails(context.Background(), domainID)
 	return result.Enabled, err
@@ -283,52 +351,42 @@ func (c *cloudflareProvider) getSingleRedirects(id string, domain string) ([]*mo
 		if errors.As(err, &e) {
 			return []*models.RecordConfig{}, nil
 		}
-		return nil, fmt.Errorf("failed fetching redirect rule list cloudflare: %s (%T)", err, err)
+		return nil, fmt.Errorf("failed fetching redirect rule list cloudflare: %w (%T)", err, err)
 	}
-	//var rulelist []cloudflare.RulesetRule
-	//rulelist = rules.Rules
-	//rulelist := rules.Rules
 
-	//printer.Printf("DEBUG: rules %+v\n", rules)
 	recs := []*models.RecordConfig{}
 	for _, pr := range rules.Rules {
-		//printer.Printf("DEBUG: %+v\n", pr)
-
-		var thisPr = pr
-		r := &models.RecordConfig{
-			Type:     "CLOUDFLAREAPI_SINGLE_REDIRECT",
-			Original: thisPr,
-			TTL:      1,
-		}
-		r.SetLabel("@", domain)
+		thisPr := pr
 
 		// Extract the valuables from the rule, use it to make the sr:
-		srMatcher := pr.Expression
-		srReplacement := pr.ActionParameters.FromValue.TargetURL.Expression
-		code := int(pr.ActionParameters.FromValue.StatusCode)
-		sr := newCfsrFromAPIData(srMatcher, srReplacement, code)
-		//sr.SRRRuleList = rulelist
-		//printer.Printf("DEBUG: DESCRIPTION = %v\n", pr.Description)
-		sr.SRDisplay = pr.Description
-		// printer.Printf("DEBUG: PR = %+v\n", pr)
-		// printer.Printf("DEBUG: rules = %+v\n", rules)
+		srName := pr.Description
+		srWhen := pr.Expression
+		srThen := pr.ActionParameters.FromValue.TargetURL.Expression
+		code := uint16(pr.ActionParameters.FromValue.StatusCode)
+
+		rec, err := rtypecontrol.NewRecordConfigFromRaw(rtypecontrol.FromRawOpts{
+			Type: "CLOUDFLAREAPI_SINGLE_REDIRECT",
+			TTL:  1,
+			Args: []any{srName, code, srWhen, srThen},
+			DCN:  domaintags.MakeDomainNameVarieties(domain),
+		})
+		if err != nil {
+			return nil, err
+		}
+		rec.Original = thisPr
+
+		// Store the IDs. These will be needed for update/delete operations.
+		sr := rec.F.(*cfsingleredirect.SingleRedirectConfig)
 		sr.SRRRulesetID = rules.ID
-		sr.SRRRulesetRuleID = pr.ID //correct
+		sr.SRRRulesetRuleID = pr.ID
 
-		r.CloudflareRedirect = sr
-		r.SetTarget(pr.Description)
-
-		recs = append(recs, r)
+		recs = append(recs, rec)
 	}
 
 	return recs, nil
 }
 
-func (c *cloudflareProvider) createSingleRedirect(domainID string, cfr models.CloudflareSingleRedirectConfig) error {
-
-	//printer.Printf("DEBUG: createSingleRedir: d=%v crf=%+v\n", domainID, cfr)
-	// Asumption for target:
-
+func (c *cloudflareProvider) createSingleRedirect(domainID string, cfr cfsingleredirect.SingleRedirectConfig) error {
 	newSingleRedirectRulesActionParameters := cloudflare.RulesetRuleActionParameters{}
 	newSingleRedirectRule := cloudflare.RulesetRule{}
 	newSingleRedirectRules := []cloudflare.RulesetRule{}
@@ -336,17 +394,18 @@ func (c *cloudflareProvider) createSingleRedirect(domainID string, cfr models.Cl
 	newSingleRedirect := cloudflare.UpdateEntrypointRulesetParams{}
 
 	// Preserve query string if there isn't one in the replacement.
-	preserveQueryString := !strings.Contains(cfr.SRReplacement, "?")
+	preserveQueryString := !strings.Contains(cfr.SRThen, "?")
 
 	newSingleRedirectRulesActionParameters.FromValue = &cloudflare.RulesetRuleActionParametersFromValue{}
 	// Redirect status code
 	newSingleRedirectRulesActionParameters.FromValue.StatusCode = uint16(cfr.Code)
 	// Incoming request expression
-	newSingleRedirectRules[0].Expression = cfr.SRMatcher
+	newSingleRedirectRules[0].Expression = cfr.SRWhen
 	// Redirect expression
-	newSingleRedirectRulesActionParameters.FromValue.TargetURL.Expression = cfr.SRReplacement
+	newSingleRedirectRulesActionParameters.FromValue.TargetURL.Expression = cfr.SRThen
 	// Redirect name
-	newSingleRedirectRules[0].Description = cfr.SRDisplay
+	newSingleRedirectRules[0].Description = cfr.SRName
+
 	// Rule action, should always be redirect in this case
 	newSingleRedirectRules[0].Action = "redirect"
 	// Phase should always be http_request_dynamic_redirect
@@ -360,47 +419,23 @@ func (c *cloudflareProvider) createSingleRedirect(domainID string, cfr models.Cl
 	rules, err := c.cfClient.GetEntrypointRuleset(context.Background(), cloudflare.ZoneIdentifier(domainID), "http_request_dynamic_redirect")
 	var e *cloudflare.NotFoundError
 	if err != nil && !errors.As(err, &e) {
-		return fmt.Errorf("failed fetching redirect rule list cloudflare: %s", err)
+		return fmt.Errorf("failed fetching redirect rule list cloudflare: %w", err)
 	}
 	newSingleRedirect.Rules = newSingleRedirectRules
-	newSingleRedirect.Rules = append(newSingleRedirect.Rules, rules.Rules...)
+	newSingleRedirect.Rules = append(rules.Rules, newSingleRedirect.Rules...)
 
 	_, err = c.cfClient.UpdateEntrypointRuleset(context.Background(), cloudflare.ZoneIdentifier(domainID), newSingleRedirect)
 
 	return err
 }
 
-func (c *cloudflareProvider) deleteSingleRedirects(domainID string, cfr models.CloudflareSingleRedirectConfig) error {
-
-	// This block should delete rules using the as is Cloudflare Golang lib in theory, need to debug why it isn't
-	// updatedRuleset := cloudflare.UpdateEntrypointRulesetParams{}
-	// updatedRulesetRules := []cloudflare.RulesetRule{}
-
-	// rules, err := c.cfClient.GetEntrypointRuleset(context.Background(), cloudflare.ZoneIdentifier(domainID), "http_request_dynamic_redirect")
-	// if err != nil {
-	// 	return fmt.Errorf("failed fetching redirect rule list cloudflare: %s", err)
-	// }
-
-	// for _, rule := range rules.Rules {
-	// 	if rule.ID != cfr.SRRRulesetRuleID {
-	// 		updatedRulesetRules = append(updatedRulesetRules, rule)
-	// 	} else {
-	// 		printer.Printf("DEBUG: MATCH %v : %v\n", rule.ID, cfr.SRRRulesetRuleID)
-	// 	}
-	// }
-	// updatedRuleset.Rules = updatedRulesetRules
-	// _, err = c.cfClient.UpdateEntrypointRuleset(context.Background(), cloudflare.ZoneIdentifier(domainID), updatedRuleset)
-
-	// Old Code
-
-	// rules, err := c.cfClient.GetEntrypointRuleset(context.Background(), cloudflare.ZoneIdentifier(domainID), "http_request_dynamic_redirect")
-	// if err != nil {
-	// 	return err
-	// }
-	//printer.Printf("DEBUG: CALLING API DeleteRulesetRule: SRRRulesetID=%v, cfr.SRRRulesetRuleID=%v\n", cfr.SRRRulesetID, cfr.SRRRulesetRuleID)
-
-	err := c.cfClient.DeleteRulesetRule(context.Background(), cloudflare.ZoneIdentifier(domainID), cfr.SRRRulesetID, cfr.SRRRulesetRuleID)
-	// TODO(tlim): This is terrible.  It returns an error even when it is successful.
+func (c *cloudflareProvider) deleteSingleRedirects(domainID string, cfr cfsingleredirect.SingleRedirectConfig) error {
+	err := c.cfClient.DeleteRulesetRule(context.Background(), cloudflare.ZoneIdentifier(domainID), cloudflare.DeleteRulesetRuleParams{
+		RulesetID:     cfr.SRRRulesetID,
+		RulesetRuleID: cfr.SRRRulesetRuleID,
+	},
+	)
+	// NB(tlim): Yuck. This returns an error even when it is successful. Dig into the JSON for the real status.
 	if strings.Contains(err.Error(), `"success": true,`) {
 		return nil
 	}
@@ -409,120 +444,34 @@ func (c *cloudflareProvider) deleteSingleRedirects(domainID string, cfr models.C
 }
 
 func (c *cloudflareProvider) updateSingleRedirect(domainID string, oldrec, newrec *models.RecordConfig) error {
-	// rulesetID := cfr.SRRRulesetID
-	// rulesetRuleID := cfr.SRRRulesetRuleID
-	//printer.Printf("DEBUG: UPDATE-DEL domID=%v sr=%+v\n", domainID, cfr)
-	if err := c.deleteSingleRedirects(domainID, *oldrec.CloudflareRedirect); err != nil {
+	if err := c.deleteSingleRedirects(domainID, *oldrec.F.(*cfsingleredirect.SingleRedirectConfig)); err != nil {
 		return err
 	}
-	//printer.Printf("DEBUG: UPDATE-CREATE domID=%v sr=%+v\n", domainID, newrec.CloudflareRedirect)
-	return c.createSingleRedirect(domainID, *newrec.CloudflareRedirect)
-}
-
-func (c *cloudflareProvider) getPageRules(id string, domain string) ([]*models.RecordConfig, error) {
-	rules, err := c.cfClient.ListPageRules(context.Background(), id)
-	if err != nil {
-		return nil, fmt.Errorf("failed fetching page rule list cloudflare: %s", err)
-	}
-	recs := []*models.RecordConfig{}
-	for _, pr := range rules {
-		// only interested in forwarding rules. Lets be very specific, and skip anything else
-		if len(pr.Actions) != 1 || len(pr.Targets) != 1 {
-			continue
-		}
-		if pr.Actions[0].ID != "forwarding_url" {
-			continue
-		}
-		value := pr.Actions[0].Value.(map[string]interface{})
-		var thisPr = pr
-		r := &models.RecordConfig{
-			Type:     "PAGE_RULE",
-			Original: thisPr,
-			TTL:      1,
-		}
-		r.SetLabel("@", domain)
-		code := intZero(value["status_code"])
-		raw := fmt.Sprintf("%s,%s,%d,%d", // $FROM,$TO,$PRIO,$CODE
-			pr.Targets[0].Constraint.Value,
-			value["url"],
-			pr.Priority,
-			code)
-		r.SetTarget(raw)
-
-		cr, err := newCfsrFromUserInput(raw, code, pr.Priority)
-		if err != nil {
-			return nil, err
-		}
-		r.CloudflareRedirect = cr
-
-		recs = append(recs, r)
-	}
-	return recs, nil
-}
-
-func (c *cloudflareProvider) deletePageRule(recordID, domainID string) error {
-	return c.cfClient.DeletePageRule(context.Background(), domainID, recordID)
-}
-
-func (c *cloudflareProvider) updatePageRule(recordID, domainID string, cfr models.CloudflareSingleRedirectConfig) error {
-	// maybe someday?
-	//c.apiProvider.UpdatePageRule(context.Background(), domainId, recordID, )
-	if err := c.deletePageRule(recordID, domainID); err != nil {
-		return err
-	}
-	return c.createPageRule(domainID, cfr)
-}
-
-func (c *cloudflareProvider) createPageRule(domainID string, cfr models.CloudflareSingleRedirectConfig) error {
-	//printer.Printf("DEBUG: called createPageRule(%s, %+v)\n", domainID, cfr)
-	// from to priority code
-	// parts := strings.Split(target, ",")
-	// priority, _ := strconv.Atoi(parts[2])
-	// code, _ := strconv.Atoi(parts[3])
-	// printer.Printf("DEBUG: pr.PageRule target = %v\n", target)
-	// printer.Printf("DEBUG: pr.PageRule target = %v\n", parts[0])
-	// printer.Printf("DEBUG: pr.PageRule url    = %v\n", parts[1])
-	// printer.Printf("DEBUG: pr.PageRule code   = %v\n", code)
-	priority := cfr.PRPriority
-	code := cfr.Code
-	matcher := cfr.PRMatcher
-	replacement := cfr.PRReplacement
-	pr := cloudflare.PageRule{
-		Status:   "active",
-		Priority: priority,
-		Targets: []cloudflare.PageRuleTarget{
-			{Target: "url", Constraint: pageRuleConstraint{Operator: "matches", Value: matcher}},
-		},
-		Actions: []cloudflare.PageRuleAction{
-			{ID: "forwarding_url", Value: &pageRuleFwdInfo{
-				StatusCode: code,
-				URL:        replacement,
-			}},
-		},
-	}
-	//printer.Printf("DEBUG: createPageRule pr=%+v\n", pr)
-	_, err := c.cfClient.CreatePageRule(context.Background(), domainID, pr)
-	return err
+	return c.createSingleRedirect(domainID, *newrec.F.(*cfsingleredirect.SingleRedirectConfig))
 }
 
 func (c *cloudflareProvider) getWorkerRoutes(id string, domain string) ([]*models.RecordConfig, error) {
 	res, err := c.cfClient.ListWorkerRoutes(context.Background(), cloudflare.ZoneIdentifier(id), cloudflare.ListWorkerRoutesParams{})
 	if err != nil {
-		return nil, fmt.Errorf("failed fetching worker route list cloudflare: %s", err)
+		return nil, fmt.Errorf("failed fetching worker route list cloudflare: %w", err)
 	}
 
 	recs := []*models.RecordConfig{}
 	for _, pr := range res.Routes {
-		var thisPr = pr
+		thisPr := pr
 		r := &models.RecordConfig{
 			Type:     "WORKER_ROUTE",
 			Original: thisPr,
 			TTL:      1,
 		}
 		r.SetLabel("@", domain)
-		r.SetTarget(fmt.Sprintf("%s,%s", // $PATTERN,$SCRIPT
+		err := r.SetTarget(fmt.Sprintf("%s,%s", // $PATTERN,$SCRIPT
 			pr.Pattern,
 			pr.ScriptName))
+		if err != nil {
+			return nil, err
+		}
+
 		recs = append(recs, r)
 	}
 	return recs, nil
@@ -534,9 +483,6 @@ func (c *cloudflareProvider) deleteWorkerRoute(recordID, domainID string) error
 }
 
 func (c *cloudflareProvider) updateWorkerRoute(recordID, domainID string, target string) error {
-	// Causing Stack Overflow (!?)
-	// return c.updateWorkerRoute(recordID, domainID, target)
-
 	if err := c.deleteWorkerRoute(recordID, domainID); err != nil {
 		return err
 	}
@@ -557,17 +503,3 @@ func (c *cloudflareProvider) createWorkerRoute(domainID string, target string) e
 	_, err := c.cfClient.CreateWorkerRoute(context.Background(), cloudflare.ZoneIdentifier(domainID), wr)
 	return err
 }
-
-// https://github.com/dominikh/go-tools/issues/1137 which is a dup of
-// https://github.com/dominikh/go-tools/issues/810
-//
-//lint:ignore U1000 false positive due to
-type pageRuleConstraint struct {
-	Operator string `json:"operator"`
-	Value    string `json:"value"`
-}
-
-type pageRuleFwdInfo struct {
-	URL        string `json:"url"`
-	StatusCode int    `json:"status_code"`
-}
