@@ -1,6 +1,7 @@
 package cscglobal
 
 import (
+	"errors"
 	"fmt"
 	"sort"
 	"strings"
@@ -8,7 +9,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/models"
 )
 
-// GetRegistrarCorrections gathers corrections that would being n to match dc.
+// GetRegistrarCorrections gathers corrections that would bring n to match dc.
 func (client *providerClient) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
 	nss, err := client.getNameservers(dc.Name)
 	if err != nil {
@@ -21,7 +22,7 @@ func (client *providerClient) GetRegistrarCorrections(dc *models.DomainConfig) (
 		if ns.Name[len(ns.Name)-1] == '.' {
 			// When this code was written ns.Name never included a single trailing dot.
 			// If that changes, the code should change too.
-			return nil, fmt.Errorf("name server includes a trailing dot, has the API changed?")
+			return nil, errors.New("name server includes a trailing dot, has the API changed?")
 		}
 		expected = append(expected, ns.Name)
 	}
