@@ -7,11 +7,11 @@ import (
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
+	dnsutilv1 "github.com/miekg/dns/dnsutil"
 )
 
 // nativeToRecord takes a DNS record from deSEC and returns a native RecordConfig struct.
 func nativeToRecords(n resourceRecord, origin string) (rcs []*models.RecordConfig) {
-
 	// deSEC returns all the values for a given label/rtype pair in each
 	// resourceRecord.  In other words, if there are multiple A
 	// records for a label, all the IP addresses are listed in
@@ -35,15 +35,15 @@ func nativeToRecords(n resourceRecord, origin string) (rcs []*models.RecordConfi
 	return rcs
 }
 
-func recordsToNative(rcs []*models.RecordConfig) []resourceRecord {
+func recordsToNative(rcs []*models.RecordConfig, origin string) []resourceRecord {
 	// Take a list of RecordConfig and return an equivalent list of resourceRecord.
 	// deSEC requires one resourceRecord for each label:key tuple, therefore we
 	// might collapse many RecordConfig into one resourceRecord.
 
-	var keys = map[models.RecordKey]*resourceRecord{}
+	keys := map[models.RecordKey]*resourceRecord{}
 	var zrs []resourceRecord
 	for _, r := range rcs {
-		label := r.GetLabel()
+		label := dnsutilv1.TrimDomainName(r.GetLabel(), origin)
 		if label == "@" {
 			label = ""
 		}
@@ -67,7 +67,6 @@ func recordsToNative(rcs []*models.RecordConfig) []resourceRecord {
 					zr.TTL = r.TTL
 				}
 			}
-
 		}
 	}
 
