@@ -7,11 +7,12 @@ import (
 	"encoding/hex"
 	"encoding/json"
 	"fmt"
-	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 	"net/http"
 	"net/http/httputil"
 	"strings"
 	"time"
+
+	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 )
 
 const (
@@ -50,7 +51,7 @@ func (restApi *dnsMadeEasyRestAPI) singleDomainGet(domainID int) (*singleDomainR
 	}
 
 	res := &singleDomainResponse{}
-	_, err := restApi.sendRequest(req, &res)
+	_, err := restApi.sendRequest(req, res)
 	if err != nil {
 		return nil, err
 	}
@@ -65,7 +66,7 @@ func (restApi *dnsMadeEasyRestAPI) multiDomainGet() (*multiDomainResponse, error
 	}
 
 	res := &multiDomainResponse{}
-	_, err := restApi.sendRequest(req, &res)
+	_, err := restApi.sendRequest(req, res)
 	if err != nil {
 		return nil, err
 	}
@@ -80,7 +81,7 @@ func (restApi *dnsMadeEasyRestAPI) recordGet(domainID int) (*recordResponse, err
 	}
 
 	res := &recordResponse{}
-	_, err := restApi.sendRequest(req, &res)
+	_, err := restApi.sendRequest(req, res)
 	if err != nil {
 		return nil, err
 	}
@@ -101,7 +102,7 @@ func (restApi *dnsMadeEasyRestAPI) singleDomainCreate(data singleDomainRequestDa
 	}
 
 	res := &singleDomainResponse{}
-	_, err = restApi.sendRequest(req, &res)
+	_, err = restApi.sendRequest(req, res)
 	if err != nil {
 		return nil, err
 	}
@@ -122,7 +123,7 @@ func (restApi *dnsMadeEasyRestAPI) multiRecordCreate(domainID int, data []record
 	}
 
 	res := &[]recordResponseDataEntry{}
-	_, err = restApi.sendRequest(req, &res)
+	_, err = restApi.sendRequest(req, res)
 	if err != nil {
 		return nil, err
 	}
@@ -185,11 +186,12 @@ func (restApi *dnsMadeEasyRestAPI) createRequest(request *apiRequest) (*http.Req
 	var req *http.Request
 	var err error
 
-	if request.method == "PUT" || request.method == "POST" {
+	switch request.method {
+	case "PUT", "POST":
 		req, err = http.NewRequest(request.method, url, bytes.NewBuffer([]byte(request.data)))
-	} else if request.method == "GET" || request.method == "DELETE" {
+	case "GET", "DELETE":
 		req, err = http.NewRequest(request.method, url, nil)
-	} else {
+	default:
 		return nil, fmt.Errorf("unknown API request method in DNSMADEEASY REST API: %s", request.method)
 	}
 
@@ -210,10 +212,10 @@ func (restApi *dnsMadeEasyRestAPI) createRequest(request *apiRequest) (*http.Req
 // DNS Made Easy only allows 150 request / 5 minutes
 // backoff is the amount of time to sleep if a "Rate limit exceeded" error is received
 // It is increased up to maxBackoff after each use
-// It is reset after successful request
+// It is reset after successful request.
 var backoff = initialBackoff
 
-func (restApi *dnsMadeEasyRestAPI) sendRequest(request *apiRequest, response interface{}) (int, error) {
+func (restApi *dnsMadeEasyRestAPI) sendRequest(request *apiRequest, response any) (int, error) {
 retry:
 	req, err := restApi.createRequest(request)
 	if err != nil {
@@ -222,7 +224,7 @@ retry:
 
 	if restApi.dumpHTTPRequest {
 		dump, _ := httputil.DumpRequest(req, true)
-		printer.Printf(string(dump))
+		printer.Printf("%s", string(dump))
 	}
 
 	res, err := restApi.httpClient.Do(req)
@@ -234,7 +236,7 @@ retry:
 
 	if restApi.dumpHTTPResponse {
 		dump, _ := httputil.DumpResponse(res, true)
-		printer.Printf(string(dump))
+		printer.Printf("%s", string(dump))
 	}
 
 	if res.StatusCode < http.StatusOK || res.StatusCode >= http.StatusBadRequest {
@@ -249,10 +251,7 @@ retry:
 
 			time.Sleep(backoff)
 
-			backoff = backoff + (backoff / 2)
-			if backoff > maxBackoff {
-				backoff = maxBackoff
-			}
+			backoff = min(backoff+(backoff/2), maxBackoff)
 
 			goto retry
 		}
@@ -263,7 +262,7 @@ retry:
 	backoff = initialBackoff
 
 	if response != nil {
-		err = json.NewDecoder(res.Body).Decode(&response)
+		err = json.NewDecoder(res.Body).Decode(response)
 		if err != nil {
 			return res.StatusCode, err
 		}
