@@ -2,9 +2,9 @@ package domainnameshop
 
 import (
 	"encoding/json"
-	"fmt"
+	"errors"
 
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 )
 
 /**
@@ -25,9 +25,10 @@ type domainNameShopProvider struct {
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanAutoDNSSEC:          providers.Cannot(),        // Maybe there is support for it
+	providers.CanAutoDNSSEC:          providers.Cannot(), // Maybe there is support for it
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Unimplemented(), //
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Unimplemented("Needs custom implementation"), // Can possibly be implemented, needs further research
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Unimplemented(), // Seems to support but needs to be implemented
@@ -47,20 +48,23 @@ var features = providers.DocumentationNotes{
 // Register with the dnscontrol system.
 // This establishes the name (all caps), and the function to call to initialize it.
 func init() {
+	const providerName = "DOMAINNAMESHOP"
+	const providerMaintainer = "@SimenBai"
 	fns := providers.DspFuncs{
 		Initializer:   newDomainNameShopProvider,
 		RecordAuditor: AuditRecords,
 	}
 
-	providers.RegisterDomainServiceProviderType("DOMAINNAMESHOP", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // newDomainNameShopProvider creates a Domainnameshop specific DNS provider.
 func newDomainNameShopProvider(conf map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	if conf["token"] == "" {
-		return nil, fmt.Errorf("no Domainnameshop token provided")
+		return nil, errors.New("no Domainnameshop token provided")
 	} else if conf["secret"] == "" {
-		return nil, fmt.Errorf("no Domainnameshop secret provided")
+		return nil, errors.New("no Domainnameshop secret provided")
 	}
 
 	api := &domainNameShopProvider{
