@@ -1,12 +1,13 @@
 package dynadot
 
 import (
+	"errors"
 	"fmt"
 	"sort"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 )
 
 /*
@@ -21,11 +22,14 @@ Info required in `creds.json`:
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanConcur: providers.Cannot(),
+	providers.CanConcur: providers.Unimplemented(),
 }
 
 func init() {
-	providers.RegisterRegistrarType("DYNADOT", newDynadot, features)
+	const providerName = "DYNADOT"
+	const providerMaintainer = "@e-im"
+	providers.RegisterRegistrarType(providerName, newDynadot, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func newDynadot(m map[string]string) (providers.Registrar, error) {
@@ -33,7 +37,7 @@ func newDynadot(m map[string]string) (providers.Registrar, error) {
 
 	d.key = m["key"]
 	if d.key == "" {
-		return nil, fmt.Errorf("missing Dynadot key")
+		return nil, errors.New("missing Dynadot key")
 	}
 
 	return d, nil
