@@ -457,19 +457,13 @@ func (api *inwxAPI) updateNameservers(ns []string, domain string) func() error {
 // GetRegistrarCorrections is part of the registrar provider and determines if the nameservers have to be updated.
 func (api *inwxAPI) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
 	regNameservers := api.fetchRegistrationNSSet(dc.Name)
-	combined := map[string]bool{}
-	for _, ns := range dc.Nameservers {
-		combined[ns.Name] = true
-	}
-	for _, rs := range regNameservers {
-		combined[rs] = true
-	}
-	var expected []string
-	for k := range combined {
-		expected = append(expected, k)
 
+	var expected []string
+	for _, ns := range dc.Nameservers {
+		expected = append(expected, ns.Name)
 	}
 	sort.Strings(expected)
+
 	foundNameservers := strings.Join(regNameservers, ",")
 	expectedNameservers := strings.Join(expected, ",")
 
@@ -484,7 +478,7 @@ func (api *inwxAPI) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.
 	return nil, nil
 }
 
-// fetchNameserverDomains returns the domains configured in INWX nameservers
+// fetchNameserverDomains returns the domains configured in INWX nameservers.
 func (api *inwxAPI) fetchNameserverDomains() error {
 	zones := map[string]int{}
 	request := &goinwx.NameserverListRequest{}
@@ -523,7 +517,7 @@ func (api *inwxAPI) fetchRegistrationNSSet(domain string) []string {
 	return info.Nameservers
 }
 
-// EnsureZoneExists creates a zone if it does not exist
+// EnsureZoneExists creates a zone if it does not exist.
 func (api *inwxAPI) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if api.domainIndex == nil { // only pull the data once.
 		if err := api.fetchNameserverDomains(); err != nil {
