@@ -4,7 +4,9 @@
 package mythicbeasts
 
 import (
+	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -12,8 +14,11 @@ import (
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
-	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/miekg/dns"
+	"github.com/StackExchange/dnscontrol/v4/pkg/dnsrr"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
+	dnsv1 "github.com/miekg/dns"
+	"golang.org/x/oauth2"
+	"golang.org/x/oauth2/clientcredentials"
 )
 
 // mythicBeastsDefaultNS lists the default nameservers, per https://www.mythic-beasts.com/support/domains/nameservers.
@@ -24,8 +29,6 @@ var mythicBeastsDefaultNS = []string{
 
 // mythicBeastsProvider is the handle for this provider.
 type mythicBeastsProvider struct {
-	secret string
-	keyID  string
 	client *http.Client
 }
 
@@ -33,7 +36,7 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanUseAlias:            providers.Cannot(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
@@ -47,24 +50,33 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "MYTHICBEASTS"
+	const providerMaintainer = "@tomfitzhenry"
 	fns := providers.DspFuncs{
 		Initializer:   newDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("MYTHICBEASTS", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func newDsp(conf map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	if conf["keyID"] == "" {
-		return nil, fmt.Errorf("missing Mythic Beasts auth keyID")
+		return nil, errors.New("missing Mythic Beasts auth keyID")
 	}
 	if conf["secret"] == "" {
-		return nil, fmt.Errorf("missing Mythic Beasts auth secret")
+		return nil, errors.New("missing Mythic Beasts auth secret")
+	}
+	// Use https://www.mythic-beasts.com/support/api/auth
+	cfg := clientcredentials.Config{
+		ClientID:     conf["keyID"],
+		ClientSecret: conf["secret"],
+		TokenURL:     "https://auth.mythic-beasts.com/login",
+		Scopes:       []string{"client_credentials"},
+		AuthStyle:    oauth2.AuthStyleInHeader,
 	}
 	return &mythicBeastsProvider{
-		keyID:  conf["keyID"],
-		secret: conf["secret"],
-		client: &http.Client{},
+		client: cfg.Client(context.Background()),
 	}, nil
 }
 
@@ -73,8 +85,6 @@ func (n *mythicBeastsProvider) httpRequest(method, url string, body io.Reader) (
 	if err != nil {
 		return nil, err
 	}
-	// https://www.mythic-beasts.com/support/api/auth
-	req.SetBasicAuth(n.keyID, n.secret)
 	req.Header.Add("Content-Type", "text/dns")
 	req.Header.Add("Accept", "text/dns")
 	return n.client.Do(req)
@@ -94,10 +104,10 @@ func (n *mythicBeastsProvider) GetZoneRecords(domain string, meta map[string]str
 }
 
 func zoneFileToRecords(r io.Reader, origin string) (models.Records, error) {
-	zp := dns.NewZoneParser(r, origin, origin)
+	zp := dnsv1.NewZoneParser(r, origin, origin)
 	var records []*models.RecordConfig
 	for rr, ok := zp.Next(); ok; rr, ok = zp.Next() {
-		rec, err := models.RRtoRC(rr, origin)
+		rec, err := dnsrr.RRtoRC(rr, origin)
 		if err != nil {
 			return nil, err
 		}
@@ -111,11 +121,12 @@ func zoneFileToRecords(r io.Reader, origin string) (models.Records, error) {
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (n *mythicBeastsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, error) {
-	msgs, changes, err := diff2.ByZone(actual, dc, nil)
+func (n *mythicBeastsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, int, error) {
+	result, err := diff2.ByZone(actual, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
+	msgs, changes, actualChangeCount := result.Msgs, result.HasChanges, result.ActualChangeCount
 
 	var corrections []*models.Correction
 	if changes {
@@ -124,9 +135,9 @@ func (n *mythicBeastsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig
 				Msg: strings.Join(msgs, "\n"),
 				F: func() error {
 					var b strings.Builder
-					for _, record := range dc.Records {
+					for _, record := range result.DesiredPlus {
 						switch rr := record.ToRR().(type) {
-						case *dns.SSHFP:
+						case *dnsv1.SSHFP:
 							// "Hex strings [for SSHFP] must be in lower-case", per Mythic Beasts API docs.
 							// miekg's DNS outputs uppercase: https://github.com/miekg/dns/blob/48f38ebef989eedc6b57f1869ae849ccc8f5fe29/types.go#L988
 							fmt.Fprintf(&b, "%s %d %d %s\n", rr.Header().String(), rr.Algorithm, rr.Type, strings.ToLower(rr.FingerPrint))
@@ -148,7 +159,7 @@ func (n *mythicBeastsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig
 			})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // GetNameservers returns the nameservers for a domain.
