@@ -11,7 +11,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	nc "github.com/billputer/go-namecheap"
 	"golang.org/x/net/publicsuffix"
 )
@@ -144,99 +144,9 @@ func (n *namecheapProvider) GetZoneRecords(domain string, meta map[string]string
 		}
 	}
 
-	// Copying this from GetDomainCorrections.  This seems redundant
-	// with what toRecords() does.  Leaving it out.
-	// 	for _, r := range records.Hosts {
-	// 		if r.Type == "SOA" {
-	// 			continue
-	// 		}
-	// 		rec := &models.RecordConfig{
-	// 			Type:         r.Type,
-	// 			TTL:          uint32(r.TTL),
-	// 			MxPreference: uint16(r.MXPref),
-	// 			Original:     r,
-	// 		}
-	// 		rec.SetLabel(r.Name, dc.Name)
-	// 		switch rtype := r.Type; rtype { // #rtype_variations
-	// 		case "TXT":
-	// 			rec.SetTargetTXT(r.Address)
-	// 		case "CAA":
-	// 			rec.SetTargetCAAString(r.Address)
-	// 		default:
-	// 			rec.SetTarget(r.Address)
-	// 		}
-	// 		actual = append(actual, rec)
-	// 	}
-
 	return toRecords(records, domain)
 }
 
-// // GetDomainCorrections returns the corrections for the domain.
-// func (n *namecheapProvider) GetDomainCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
-// 	dc.Punycode()
-// 	sld, tld := splitDomain(dc.Name)
-// 	var records *nc.DomainDNSGetHostsResult
-// 	var err error
-// 	doWithRetry(func() error {
-// 		records, err = n.client.DomainsDNSGetHosts(sld, tld)
-// 		return err
-// 	})
-// 	if err != nil {
-// 		return nil, err
-// 	}
-
-// 	var actual []*models.RecordConfig
-
-// 	// namecheap does not allow setting @ NS with basic DNS
-// 	dc.Filter(func(r *models.RecordConfig) bool {
-// 		if r.Type == "NS" && r.GetLabel() == "@" {
-// 			if !strings.HasSuffix(r.GetTargetField(), "registrar-servers.com.") {
-// 				printer.Println("\n", r.GetTargetField(), "Namecheap does not support changing apex NS records. Skipping.")
-// 			}
-// 			return false
-// 		}
-// 		return true
-// 	})
-
-// 	// namecheap has this really annoying feature where they add some parking records if you have no records.
-// 	// This causes a few problems for our purposes, specifically the integration tests.
-// 	// lets detect that one case and pretend it is a no-op.
-// 	if len(dc.Records) == 0 && len(records.Hosts) == 2 {
-// 		if records.Hosts[0].Type == "CNAME" &&
-// 			strings.Contains(records.Hosts[0].Address, "parkingpage") &&
-// 			records.Hosts[1].Type == "URL" {
-// 			return nil, nil
-// 		}
-// 	}
-
-// 	for _, r := range records.Hosts {
-// 		if r.Type == "SOA" {
-// 			continue
-// 		}
-// 		rec := &models.RecordConfig{
-// 			Type:         r.Type,
-// 			TTL:          uint32(r.TTL),
-// 			MxPreference: uint16(r.MXPref),
-// 			Original:     r,
-// 		}
-// 		rec.SetLabel(r.Name, dc.Name)
-// 		switch rtype := r.Type; rtype { // #rtype_variations
-// 		case "TXT":
-// 			rec.SetTargetTXT(r.Address)
-// 		case "CAA":
-// 			rec.SetTargetCAAString(r.Address)
-// 		default:
-// 			rec.SetTarget(r.Address)
-// 		}
-// 		actual = append(actual, rec)
-// 	}
-
-// 	// Normalize
-// 	models.PostProcessRecords(actual)
-
-// 	return n.GetZoneRecordsCorrections(dc, actual)
-// }
-
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
 func (n *namecheapProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, int, error) {
 	// namecheap does not allow setting @ NS with basic DNS
@@ -383,12 +293,14 @@ func (n *namecheapProvider) GetRegistrarCorrections(dc *models.DomainConfig) ([]
 	}
 	sort.Strings(info.DNSDetails.Nameservers)
 	found := strings.Join(info.DNSDetails.Nameservers, ",")
+
 	desiredNs := []string{}
 	for _, d := range dc.Nameservers {
 		desiredNs = append(desiredNs, d.Name)
 	}
 	sort.Strings(desiredNs)
 	desired := strings.Join(desiredNs, ",")
+
 	if found != desired {
 		parts := strings.SplitN(dc.Name, ".", 2)
 		sld, tld := parts[0], parts[1]
